#ifndef _DOCLIST_PDLCOUNT_H
#define _DOCLIST_PDLCOUNT_H

#include <sdsl/bit_vectors.hpp>

#include "utils.h"


void buildPDLCount(const std::string& base_name, bool print);

class PDLCount
{
  public:
    const static std::string EXTENSION; // .pdlcount

    const static usint RANGE_BLOCK_SIZE = 32;
    const static usint COUNT_BLOCK_SIZE = 32;

    typedef enum { st_unfinished, st_error, st_ok } status_type;

    PDLCount(const RLCSA& _rlcsa, const std::string& base_name);
    ~PDLCount();

    inline bool isOk() const { return (this->status == st_ok); }

    void writeTo(const std::string& base_name) const;
    usint reportSize() const;

    usint count(const std::string& pattern) const;
    usint count(pair_type sa_range) const;

  private:
    const RLCSA&     rlcsa;
    status_type      status;

    CSA::DeltaVector* leaf_ranges;
    CSA::Array*       counts;

    sdsl::bit_vector                tree;
    sdsl::bit_vector::rank_1_type   tree_rank;
    sdsl::select_support_mcl<01, 2> tree_select;

    // These are not allowed.
    PDLCount();
    PDLCount(const PDLCount&);
    PDLCount& operator = (const PDLCount&);
};


#endif  // _DOCLIST_PDLCOUNT_H
